import 'dart:async';
import 'util_list_sort.dart';
import 'package:flutter/material.dart';

/// widget added inside widget tree that exposes a stream to access to listen to mediaquerystate changes
class MediaQueryWatcher extends InheritedWidget {
  MediaQueryWatcher({
    Key? key,
    required this.widthWatchThreshholds,
    required this.heightWatchThreshholds,
    required Widget child,
  }) : super(key: key, child: _MediaQueryWatcherInternal(child)) {
    // ### SORT thresholds ###
    widthWatchThreshholds = widthWatchThreshholds..quickSort();
    heightWatchThreshholds = heightWatchThreshholds..quickSort();
    // ### listen to mediaquerystream ###
    _mediaQueryInputStreamController.stream.distinct().listen((Size _size) {
      for (var i = 0; i < heightWatchThreshholds.length; i++) {
        if (i == heightWatchThreshholds.length - 1) {
          inputSink.add(_size);
          heightWatchIndex = i;
          break;
        }
        if (_size.width < heightWatchThreshholds[i] &&
            _size.width > heightWatchThreshholds[i + 1]) {
          if (i != heightWatchIndex) {
            inputSink.add(_size);
            heightWatchIndex = i;
          }
        }
      }
    });
  }

  /// widthwatchindex
  int? widthWatchIndex;

  /// heightwatchindex
  int? heightWatchIndex;

  /// widthWatchThreshholds
  List<double> widthWatchThreshholds;

  /// heightWatchThreshholds
  List<double> heightWatchThreshholds;

  final StreamController<Size> _mediaQueryInputStreamController =
      StreamController<Size>.broadcast();

  /// should not be used but sink to put size objects
  EventSink<Size> get inputSink => _mediaQueryInputStreamController.sink;

  /// listen to every size change width + height
  Stream<Size> get sizeStream => _mediaQueryInputStreamController.stream;

  /// listen to width changes
  Stream<double> get widthStream => _mediaQueryInputStreamController.stream
      .map<double>((_size) => _size.width);

  /// listen to height changes
  Stream<double> get heightStream => _mediaQueryInputStreamController.stream
      .map<double>((_size) => _size.height);

  /// access the mediaquerywatcher by .of()
  static MediaQueryWatcher of(BuildContext context) {
    return context.dependOnInheritedWidgetOfExactType<MediaQueryWatcher>()!;
  }

  //// updateshouldnotify
  @override
  bool updateShouldNotify(covariant MediaQueryWatcher oldWidget) {
    return false;
  }

  /// deactivate the widgets listening function
  void deactivate() {
    _mediaQueryInputStreamController.close();
  }
}

class _MediaQueryWatcherInternal extends StatelessWidget {
  const _MediaQueryWatcherInternal(this.child);
  final Widget child;
  @override
  Widget build(BuildContext context) {
    final Size _size = MediaQuery.of(context).size;
    MediaQueryWatcher.of(context).inputSink.add(
          _size,
        );
    return child;
  }
}
